import requests, json, datetime, sys, logging
from .utils import Utils
from .models.product import Product
from .models.product import StoreProduct
from .models.product import ProductCategorie
from .models.product import ProductCategorieChild
from .models.product import MarketplaceCategory
from .models.order import OrderProductPackageDimensions
from .models.order import OrderProduct
from .models.order import OrderAddress
from .models.order import OrderPayment
from .models.order import OrderCustomerAddress
from .models.order import OrderCustomer
from .models.order import OrderShippingAddress
from .models.order import OrderShippingItem
from .models.order import OrderShipping
from .models.order import InvoiceItem
from .models.order import Invoice
from .models.order import Order

logformat = "%(asctime)s %(levelname)s %(module)s - %(funcName)s: %(message)s"
datefmt = "%m-%d %H:%M"
logging.basicConfig(filename="last_execution.log", level=logging.INFO, filemode="w",
					format=logformat, datefmt=datefmt)
stream_handler = logging.StreamHandler(sys.stderr)
stream_handler.setFormatter(logging.Formatter(fmt=logformat, datefmt=datefmt))
logger = logging.getLogger("app")
logger.addHandler(stream_handler)
logger.info("information")

class HubController:
	@classmethod
	def authenticate(self, server_url='', user='', passwd=''):
		# create headers
		headers = dict()
		headers['Content-Type'] = 'application/json'

		# authenticate payload
		payload = dict()
		payload['username'] = user
		payload['password'] = passwd

		# do login request
		r = requests.post(f'{server_url}/signin/', headers=headers, data=json.dumps(payload))
		
		# catch error
		if r.status_code != 200:
			raise Exception(f'Failed to login on NappHUB...try again... {r.content.decode("utf-8")}')

		# get and return token
		token = json.loads(r.content.decode('utf8'))['token']
		return token

	@classmethod
	def get_all_attributes(self, server_url='', token=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/attributes/', headers=headers, timeout=60)
		r = r.content.decode('utf-8')
		
		# create products object
		attributes = None
		
		# check if exists products on this server
		if r and r != 'null':
			attributes = json.loads(r)
		
		# return
		return attributes

	@classmethod
	def get_product_by_id(self, server_url='', token='', product_id=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/products/{product_id}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')
		
		# create products object
		product = None
		
		# check if exists products on this server
		if r and r != 'null':
			product = json.loads(r)
		
		# return
		return product

	@classmethod
	def get_all_products(self, server_url='', token=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/products/', headers=headers, timeout=60)
		r = r.content.decode('utf-8')
		
		# create products object
		products = None
		
		# check if exists products on this server
		if r and r != 'null':
			products = json.loads(r)
		
		# return
		return products

	@classmethod
	def get_all_store_products(self, server_url='', token='', store_id=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/storeProductsByStore/{store_id}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')
		
		# create products object
		store_products = None
		
		# check if exists products on this server
		if r and r != 'null':
			store_products = json.loads(r)
		
		# return
		return store_products

	@classmethod
	def get_product_by_ean(self, server_url='', token='', ean=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/productsByEan/?ean={ean}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')

		# create id object
		product = None
		
		# find store product
		if r and r != 'null':
			product = json.loads(r)
		
		# return
		return product

	@classmethod
	def get_store_product_by_id(self, server_url='', token='', id='', store_id=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/storeProducts/{id}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')
		
		# create id object
		store_product = None
		
		# find store product
		if r and r != 'null':
			store_product = json.loads(r)
		
		# return
		return store_product

	@classmethod
	def get_store_product_by_erpCode(self, server_url='', token='', erp_code='', store_id=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = requests.get(f'{server_url}/storeProductsByErpCodeAndStore/{erp_code}/{store_id}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')

		# create id object
		store_product = None
		
		# find store product
		if r and r != 'null':
			store_product = json.loads(r)
		
		# return
		return store_product

	@classmethod
	def get_store_product_by_productCodeOrEan(self, server_url='', token='', store_id='', product_code='', ean=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# do request
		r = None
		if product_code != '':
			r = requests.get(f'{server_url}/storeProducts/?storeId={store_id}&productCode={product_code}', headers=headers, timeout=60)
		elif ean != '':
			r = requests.get(f'{server_url}/storeProducts/?storeId={store_id}&ean={ean}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')

		# create object
		store_product = None
		
		# find store product
		if r and r != 'null':
			store_product = json.loads(r)['data']
		
		# return
		return store_product

	@classmethod
	def get_store_product_marketplace(self, server_url='', token='', marketplace_id='', store_id='', store_product_id=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		r = requests.get(f'{server_url}/storeProductsMarketplace/?marketplaceId={marketplace_id}&storeId={store_id}&storeProductId={store_product_id}', headers=headers, timeout=60)
		r = r.content.decode('utf-8')

		store_product_marketplace = None

		# find store product marketplace
		if r != None and r != 'null':
			store_product_marketplace = json.loads(r)['data']

		# return
		return store_product_marketplace

	@classmethod
	def create_store_product_marketplace(self, server_url, token, store_product_marketplace=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create 'store product marketplace' payload
		payload_store_product_marketplace = json.dumps(store_product_marketplace, ensure_ascii=False)

		# do request on POST/storeProductsMarketplace
		r = requests.post(f'{server_url}/storeProductsMarketplace/', headers=headers, data=payload_store_product_marketplace)
		logger.info(f'Creating store product marketplace...{r.status_code} - {r.content.decode("utf-8")}')

	@classmethod
	def update_store_product_marketplace(self, server_url, token, store_product_marketplace=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create 'store product marketplace' payload
		payload_store_product_marketplace = json.dumps(store_product_marketplace, ensure_ascii=False)

		# do request on POST/storeProductsMarketplace
		r = requests.put(f'{server_url}/storeProductsMarketplace/', headers=headers, data=payload_store_product_marketplace)
		logger.info(f'Updating store product marketplace...{r.status_code} - {r.content.decode("utf-8")}')

	@classmethod
	def update_store_product(self, server_url, token, inventory, store_product_id, product_id):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# set hub product ID on object store product
		inventory.id = store_product_id
		inventory.productId = Utils.create_values(product_id, 'Int64')

		# create store product update payload
		payload_store_product = json.dumps(inventory.__dict__, ensure_ascii=False)

		# do request on PUT/storeProducts
		r = requests.put(f'{server_url}/storeProducts/', headers=headers, data=payload_store_product)

		# log
		logger.info(f'Updating StoreProducts ID <{inventory.id}>... {r.status_code}:{r.content.decode("utf-8")}')

	@classmethod
	def create_store_product(self, server_url, token, inventory):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create store product payload
		payload_store_product = inventory.__dict__

		# create 'store product' payload
		payload_store_product = json.dumps(payload_store_product, ensure_ascii=False)

		# do request on POST/storeProducts
		r = requests.post(f'{server_url}/storeProducts/', headers=headers, data=payload_store_product)
		logger.info(f'Creating store product...{r.status_code} - {r.content.decode("utf-8")}')

		return r.json()

	@classmethod
	def update_product(self, server_url, token, id_product, product):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create product update payload
		product = product.__dict__
		product.update({"id": int(id_product)})

		payload_product = json.dumps(product, ensure_ascii=False)
		
		# do request on PUT/storeProducts
		r = requests.put(f'{server_url}/products/', headers=headers, data=payload_product)

		# log
		logger.info(f'Updating Products... ID <{product["id"]}>... {r.status_code}:{r.content.decode("utf-8").strip()}')

		return r.status_code

	@classmethod
	def create_category(self, server_url, token, categories=[]):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# Create request object
		r = None
		
		# Create payload_category
		payload_category = json.dumps(categories.__dict__, ensure_ascii=False)
	
		r = requests.post(f'{server_url}/categories/', headers=headers, data=payload_category)

		return r.status_code

	@classmethod
	def create_category_child(self, server_url, token, child_categories=[]):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# Create request object
		r = None
		
		# Create payload_category
		payload_child_category = json.dumps(child_categories.__dict__, ensure_ascii=False)
	
		r = requests.post(f'{server_url}/categories/', headers=headers, data=payload_child_category)

		r.status_code

	@classmethod
	def create_marketplace_category(self, server_url, token, marketplace_categories=[]):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# Create request object
		r = None
		
		# Create payload_category
		payload_marketpace_category = json.dumps(marketplace_categories.__dict__, ensure_ascii=False)
	
		r = requests.post(f'{server_url}/marketplaceCategories/', headers=headers, data=payload_marketpace_category)
		
		return r.status_code

	@classmethod
	def check_brand(self, server_url, token, brand=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		logger.info('checando a marca - [{}]'.format(brand))

		# baixa as marcas de produtos já cadastradas no NappHub
		r = requests.get(f'{server_url}/productsBrandByName/?name={brand}', headers=headers)
		brand_hub = r.json()

		if brand_hub['description'] == str(brand).strip():
			logger.info("marca já está registrada no NappHub")

		else:
			logger.info("marca não existe no NappHub, enviando...")

			body = dict()
			body['description'] = brand

			r = requests.post(f'{server_url}/productsBrand/', headers=headers, data=json.dumps(body))

			if r.status_code == 200:
				logger.info('a marca: [{}] foi registrada com sucesso'.format(brand))
			else:
				logger.error('Erro ao registrar a marca [{} - code: {} - text: {}]'.format(brand, r.status_code, r.text))

	@classmethod
	def check_marketplace_invetory(self, server_url='', token='', inventory='', marketplace_id='', store_id='', store_product_id=0, mkt_category=''):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		r = requests.get(f'{server_url}/storeProductsMarketplace/?marketplaceId={marketplace_id}&storeId={store_id}&storeProductId={store_product_id}', headers=headers)

		if r['data']:
			logger.info("inventário de marketplace já existe, atualizando...")

			'''
			self.update_store_product_marketplace(

			)
			'''

		else:
			logger.info("inventário de marketplace não existe, enviando... ")

			'''
			self.create_store_product_marketplace(


			)

			'''
			
	@classmethod
	def check_invetory(self, server_url, token, store_products=[], store_id=0, marketplace_id=0, use_ean=False, use_sku=False, use_mkt=False):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create arrays
		hub_products = []
		hub_products_not_found = []

		response = None

		for store_product in store_products:
			invetaryEan = store_product.productEan['String']
			invetaryCode = store_product.productCode['String']

			# Check if use ean or sku to find product
			if use_ean:
				r = requests.get(f'{server_url}/storeProducts/?ean={invetaryEan}&storeId={store_id}', headers=headers)
				r = r.json()
			elif use_sku:
				r = requests.get(f'{server_url}/storeProducts/?productCode={invetaryCode}&storeId={store_id}', headers=headers)
				r = r.json()

			if r['data']:
				store_product_hub = r['data'][0]
				logger.info('inventário já existe, atualizando...')

				self.update_store_product(
					server_url=server_url, 
					token=token,
					inventory=store_product,
					store_product_id=store_product_hub['id'], 
					product_id=store_product_hub['productId']['Int64']
				)

				store_product_id = store_product_hub['id']

			else:
				logger.info('inventário não existe, enviando...')
			
				response = self.create_store_product(
					server_url=server_url,
					token=token,
					inventory=store_product
				)

				store_product_id = response
				
			if use_mkt:
				self.check_marketplace_invetory(
					store_product_id = store_product_id,
					marketplace_id = marketplace_id,
					store_id = store_id,
					mkt_category=categorie,
					inventory=store_product
				)

	@classmethod
	def create_product(self, server_url='', token='', store_id='', products=[], use_ean=False, use_sku=False):
		  # create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create arrays
		hub_products = []
		hub_products_not_found = []

		response = None

		# loop in all integration produtcs
		for product in products:
			# create request object
			r = None

			# get product ids string,
			productEan = product.productEan['String']
			productCode = product.productCode['String']
			
			# Check if use ean or sku to find product
			if use_ean:
				r = requests.get(f'{server_url}/productsByEan/?ean={productEan}', headers=headers)
			elif use_sku:
				r = requests.get(f'{server_url}/productsByProductCode/?code={productCode}&storeId={store_id}', headers=headers)

			if r:
				# find hub product id based on integration product
				hub_product = json.loads(r.content.decode('utf-8'))
				hub_product_id = hub_product['id'] 
				hub_product_name = hub_product['name']
				hub_product_desc = hub_product['description']

				# check if product id exists on napp hub
				if hub_product_id != 0:
					# log
					logger.warning(f'Produto já existe no NappHub [<{productEan}>/<{productCode}>]')

					# valida se mudou algo no produto, antes de chamar a função de update
					if str(hub_product_name) != str(product.name) or str(hub_product_desc) != str(product.description):
						response = self.update_product(
							server_url=server_url,
							token=token,
							id_product=hub_product_id,
							product=product)

						logger.info(f"Produto foi modificado: <{productEan}>/<{productCode}>")
				else:
					# clear 'store products' from 'products' and create a new payload
					product = product.__dict__
					product.pop('storeProduct')

					# check exist's brand
					self.check_brand(server_url=server_url, token=token, brand=product['productBrandName']['String'])
					payload_product = json.dumps(product, ensure_ascii=False)
					
					# do request on POST products
					response = requests.post(f'{server_url}/products/', headers=headers, data=payload_product)
					logger.info(f'Criando o produto... [{"sucesso ao registrar o produto (200)" if response.status_code == 200 else response.status_code}]')

		return response

	@classmethod
	def create_products_and_store_products(self, server_url='', token='', store_id='', products=[], use_ean=False, use_sku=False, update_product=False, update_store_product=False):
		# create headers
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'

		# create arrays
		hub_products = []
		hub_products_not_found = []

		# loop in all integration produtcs
		for product in products:
			# create request object
			r = None

			# get product ids string
			productEan = product.productEan['String']
			productCode = product.productCode['String']
			
			# Check if use ean or sku to find product
			if use_ean:
				r = requests.get(f'{server_url}/productsByEan/?ean={productEan}', headers=headers)
			elif use_sku:
				r = requests.get(f'{server_url}/productsByProductCode/?code={productCode}&storeId={store_id}', headers=headers)

			# check if request exists
			if r:
				# find hub product id based on integration product
				hub_product = json.loads(r.content.decode('utf-8'))
				hub_product_id = hub_product['id'] 

				# check if product id exists on napp hub
				if hub_product_id != 0:
					# log
					logger.info(f'Code <{productEan}>/<{productCode}> exists on Napp HUB')

					# check update enabled
					if update_store_product:
						# find store product id
						store_product = self.get_store_product_by_erpCode(
							server_url=server_url, 
							token=token, 
							erpCode=productCode, 
							store_id=store_id)
						
						# update if exists or create
						if store_product: 
							self.update_store_product(
								server_url=server_url, 
								token=token,
								product=product,
								store_product_id=store_product['id'], 
								product_id=hub_product_id)
						else:
							self.create_store_product(
								server_url=server_url,
								token=token,
								product=product,
								product_id=hub_product_id)

					# update product
					if update_product:
						# set hub product id on object product
						product.id = hub_product_id

						# call
						self.update_product(
							server_url=server_url,
							token=token, 
							product=product)
				else:
					# create store product payload
					payload_store_product = product.storeProduct.__dict__

					# clear 'store products' from 'products' and create a new payload
					product = product.__dict__
					product['storeProduct'] = None
					payload_product = json.dumps(product, ensure_ascii=False)

					# do request on POST products
					r = requests.post(f'{server_url}/products/', headers=headers, data=payload_product)
					logger.info(f'Creating product...{r.status_code} - {r.content.decode("utf-8")}')

					if r.status_code == 200:
						# get created product id
						product_id = int(r.content.decode('utf-8'))

						# call function
						self.create_store_product(
							server_url=server_url,
							token=token,
							product=product,
							product_id=product_id)
	
	@classmethod
	def find_parents_childs(self, server_url='', token='', store_products=''):
		# create arrays and tmp last parent id
		parents = []
		products = []
		count = 1
		qty = len(store_products)
		last_parent = 0

		# loop in all store products
		for store_product in store_products:
			try:
				logger.info(f'Identify parent and child from store product: {count}/{qty}')
				# get product ID and product object
				product_id = store_product['productId']
				product = self.get_product_by_id(server_url=server_url, token=token, product_id=product_id)
				
				if product:
					# append products to list and get parent ID
					products.append({'storeProduct': store_product, 'product': product})
					parent_id = product['parentId']['Int64']

					# check if parent id is different from last parent id check
					if parent_id != last_parent:
						# get product parent object and append to list
						parent = self.get_product_by_id(server_url=server_url, token=token, product_id=product['parentId']['Int64'])
						if parent:
							parents.append({'storeProduct': store_product, 'product': parent})      
							# set latest parent id
							last_parent = parent_id
			except Exception as e:
				logger.info(f'Failed to identify parent and child... {e}')
				pass

			# count
			count += 1
		return parents, products

	@classmethod
	def get_order_by_foreign_id(self, server_url='', token='', orderForeignId='', storeId='', purchasedAt=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   
		
		url = f'{server_url}/orderByForeignId/?foreignId={orderForeignId}&storeId={storeId}&purchasedAt={purchasedAt}'
		r = requests.get(url, headers=headers)
		r = r.content.decode('utf-8')
		r = json.loads(r)

		return r

	@classmethod
	def create_order(self, server_url='', token='', order=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   

		payload = json.dumps(order)
		r = requests.post(f'{server_url}/orders/', headers=headers, data=payload)
		print(f'Creating order {order.get("orderForeignId")["String"]}...{r.status_code}')

		r = r.content.decode('utf-8')
		return r

	@classmethod
	def update_order(self, server_url='', token='', order=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   

		payload = json.dumps(order)
		r = requests.put(f'{server_url}/orders/', headers=headers, data=payload)
		print(f'Updating order {order.get("orderForeignId")["String"]}...{r.status_code}')

		r = r.content.decode('utf-8')
		return r

	@classmethod
	def create_order_invoice(self, server_url='', token='', invoice=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   

		payload = json.dumps(invoice)
		r = requests.post(f'{server_url}/invoices/', headers=headers, data=payload)
		r = r.content.decode('utf-8')
		print(f'Creating invoice {invoice.get("orderId")["Int64"]}...{r}')

	@classmethod
	def create_order_shipping(self, server_url='', token='', shipping=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   

		payload = json.dumps(shipping)
		r = requests.post(f'{server_url}/shippings/', headers=headers, data=payload)
		r = r.content.decode('utf-8')
		print(f'Creating shipping {shipping.get("orderId")["Int64"]}...{r}')

	@classmethod
	def create_order_payment(self, server_url='', token='', payment=''):
		headers = dict()
		headers['Authorization'] = f'Bearer {token}'   

		payload = json.dumps(payment)
		r = requests.post(f'{server_url}/payments/', headers=headers, data=payload)
		r = r.content.decode('utf-8')
		print(f'Creating payment {payment.get("orderId")}...{r}')
